#include <sourcemod>
#include <sdktools>

#pragma semicolon 1

#define PLUGIN_VERSION "1.5"
#define MAX_PLAYERS 64

new Handle:cvarsoundenable;
new Handle:cvarsoundlimit;
new Handle:cvarsoundwarn;
new Handle:cvarjoinexit;
new Handle:cvartimebetween;
new Handle:listfile;
new String:soundlistfile[PLATFORM_MAX_PATH];
new restrict_playing_sounds[MAX_PLAYERS+1];
new SndOn[MAX_PLAYERS+1];
new SndCount[MAX_PLAYERS+1];
new Float:LastSound[MAX_PLAYERS+1];

public Plugin:myinfo = 
{
	name = "Say Sounds",
	author = "Hell Phoenix",
	description = "Say Sounds",
	version = PLUGIN_VERSION,
	url = "http://www.charliemaurice.com/plugins/"
};

public OnPluginStart(){
	CreateConVar("sm_saysounds_version", PLUGIN_VERSION, "Say Sounds Version", FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY);
	cvarsoundenable = CreateConVar("sm_sound_enable","1","Turns Sounds On/Off",FCVAR_PLUGIN);
	cvarsoundwarn = CreateConVar("sm_sound_warn","3","Number of sounds to warn person at",FCVAR_PLUGIN);
	cvarsoundlimit = CreateConVar("sm_sound_limit","1000","Maximum sounds per person",FCVAR_PLUGIN);
	cvarjoinexit = CreateConVar("sm_join_exit","0","Play sounds when someone joins or exits the game",FCVAR_PLUGIN);
	cvartimebetween = CreateConVar("sm_time_between_sounds","2","Time between each sound trigger, 0.0 to disable checking",FCVAR_PLUGIN);
	RegAdminCmd("sm_sound_ban", Command_Sound_Ban, ADMFLAG_BAN, "sm_sound_ban <user> : Bans a player from using sounds");
	RegAdminCmd("sm_sound_unban", Command_Sound_Unban, ADMFLAG_BAN, "sm_sound_unban <user> : Unbans a player from using sounds");
	RegAdminCmd("sm_sound_reset", Command_Sound_Reset, ADMFLAG_BAN, "sm_sound_reset <user | all> : Resets sound quota for user, or everyone if all");
	RegConsoleCmd("say", Command_Say);
	RegConsoleCmd("say2", Command_InsurgencySay);
	RegConsoleCmd("say_team", Command_Say);
}

public OnMapStart(){
	CreateTimer(0.1, Load_Sounds);
}

public Action:Load_Sounds(Handle:timer){
	// precache sounds, loop through sounds
	BuildPath(Path_SM,soundlistfile,sizeof(soundlistfile),"configs/saysounds.cfg");
	if(!FileExists(soundlistfile)) {
		LogMessage("saysounds.cfg not parsed...file doesnt exist!");
	}else{
		listfile = CreateKeyValues("soundlist");
		FileToKeyValues(listfile,soundlistfile);
		KvRewind(listfile);
		KvGotoFirstSubKey(listfile);
		do{
			decl String:filelocation[255];
			decl String:dl[255];
			KvGetString(listfile, "file", filelocation, sizeof(filelocation));
			Format(dl, sizeof(filelocation), "sound/%s", filelocation);
			if(FileExists(dl)){
				PrecacheSound(filelocation, true);
				AddFileToDownloadsTable(dl);
			}
		} while (KvGotoNextKey(listfile));
	}
}

public OnClientAuthorized(client, const String:auth[]){
	if(!IsFakeClient(client)){
		if(client != 0){
			SndOn[client] = 1;
			SndCount[client] = 0;
			LastSound[client] = 0.0;
			
			if(!GetConVarInt(cvarjoinexit))
				return;
				
			decl String:filelocation[255];
			new adminonly;
			KvJumpToKey(listfile, "JoinSound");
			KvGetString(listfile, "file", filelocation, sizeof(filelocation));
			adminonly = KvGetNum(listfile, "admin");
			
			new Handle:pack;
			CreateDataTimer(0.2,Command_Play_Sound,pack);
			WritePackCell(pack, client);
			WritePackCell(pack, adminonly);
			WritePackString(pack, filelocation);
			
			SndCount[client] = 0;
		}
	}
}

public OnClientDisconnect(client){
	if(!GetConVarInt(cvarjoinexit))
		return;
		
	SndCount[client] = 0;
				
	decl String:filelocation[255];
	new adminonly;
	KvJumpToKey(listfile, "ExitSound");
	KvGetString(listfile, "file", filelocation, sizeof(filelocation));
	adminonly = KvGetNum(listfile, "admin");
	
	new Handle:pack;
	CreateDataTimer(0.2,Command_Play_Sound,pack);
	WritePackCell(pack, client);
	WritePackCell(pack, adminonly);
	WritePackString(pack, filelocation);
	
}

public Action:Command_Say(client,args){
	if(client != 0){
		// If sounds are not enabled, then skip this whole thing
		if (!GetConVarInt(cvarsoundenable))
			return Plugin_Continue;
	
		// player is banned from playing sounds
		if (restrict_playing_sounds[client])
			return Plugin_Continue;
			
		decl String:speech[128];
		decl String:clientName[64];
		GetClientName(client,clientName,64);
		GetCmdArgString(speech,sizeof(speech));
		
		new startidx = 0;
		if (speech[0] == '"'){
			startidx = 1;
			/* Strip the ending quote, if there is one */
			new len = strlen(speech);
			if (speech[len-1] == '"'){
				speech[len-1] = '\0';
			}
		}
						
		if(strcmp(speech[startidx],"!sound",false) == 0){
				if(SndOn[client] == 1){
					SndOn[client] = 0;
					PrintToChat(client,"[Funny Sounds] Funny Sounds Disabled");
				}else{
					SndOn[client] = 1;
					PrintToChat(client,"[Funny Sounds] Funny Sounds Enabled");
				}
				return Plugin_Handled;
		}
		
		if(strcmp(speech[startidx],"!wtf",false) == 0){
			List_Sounds(client);
			PrintToChat(client,"[Funny Sounds] Check your console for a list of sound triggers");
			return Plugin_Handled;
		}
			
		
		KvRewind(listfile);
		KvGotoFirstSubKey(listfile);
		decl String:buffer[255];
		decl String:filelocation[255];
		new adminonly;
		do{
			KvGetSectionName(listfile, buffer, sizeof(buffer));
			if (strcmp(speech[startidx],buffer,false) == 0){
				KvGetString(listfile, "file", filelocation, sizeof(filelocation));
				adminonly = KvGetNum(listfile, "admin");
				new Handle:pack;
				CreateDataTimer(0.1,Command_Play_Sound,pack);
				WritePackCell(pack, client);
				WritePackCell(pack, adminonly);
				WritePackString(pack, filelocation);
				break;
			}
		} while (KvGotoNextKey(listfile));
 
		return Plugin_Continue;
	}	
	return Plugin_Continue;
}

public Action:Command_InsurgencySay(client,args){
	if(client != 0){
		// If sounds are not enabled, then skip this whole thing
		if (!GetConVarInt(cvarsoundenable))
			return Plugin_Continue;
	
		// player is banned from playing sounds
		if (restrict_playing_sounds[client])
			return Plugin_Continue;
			
		decl String:speech[128];
		decl String:clientName[64];
		GetClientName(client,clientName,64);
		GetCmdArgString(speech,sizeof(speech));
		
		new startidx = 4;
		if (speech[0] == '"'){
			startidx = 5;
			/* Strip the ending quote, if there is one */
			new len = strlen(speech);
			if (speech[len-1] == '"'){
				speech[len-1] = '\0';
			}
		}
						
		if(strcmp(speech[startidx],"!sounds",false) == 0){
				if(SndOn[client] == 1){
					SndOn[client] = 0;
					PrintToChat(client,"[Funny Sounds] Sounds Disabled");
				}else{
					SndOn[client] = 1;
					PrintToChat(client,"[Funny Sounds] Sounds Enabled");
				}
				return Plugin_Handled;
		}
		
		if(strcmp(speech[startidx],"!wtf",false) == 0){
			List_Sounds(client);
			PrintToChat(client,"[Funny Sounds] Check your console for a list of sound triggers");
			return Plugin_Handled;
		}
		
			
		KvRewind(listfile);
		KvGotoFirstSubKey(listfile);
		decl String:buffer[255];
		decl String:filelocation[255];
		new adminonly;
		do{
			KvGetSectionName(listfile, buffer, sizeof(buffer));
			if (strcmp(speech[startidx],buffer,false) == 0){
				KvGetString(listfile, "file", filelocation, sizeof(filelocation));
				adminonly = KvGetNum(listfile, "admin");
				new Handle:pack;
				CreateDataTimer(0.1,Command_Play_Sound,pack);
				WritePackCell(pack, client);
				WritePackCell(pack, adminonly);
				WritePackString(pack, filelocation);
				break;
			}
		} while (KvGotoNextKey(listfile));
		
		return Plugin_Continue;
	}	
	return Plugin_Continue;
}

public Action:Command_Play_Sound(Handle:timer,Handle:pack){
	decl String:filelocation[255];
	ResetPack(pack);
	new client = ReadPackCell(pack);
	new adminonly = ReadPackCell(pack);
	ReadPackString(pack, filelocation, sizeof(filelocation));
	
	if(adminonly){
		new AdminId:aid = GetUserAdmin(client);
		if (aid == INVALID_ADMIN_ID)
			return Plugin_Handled;
	}
	
	new Float:waitTime = GetConVarFloat(cvartimebetween);
	new Float:thetime = GetGameTime();
	
	if (LastSound[client] >= thetime){
		PrintToChat(client,"[Funny Sounds] Please don't spam the sounds");
	}
	
	if ((SndCount[client] < GetConVarInt(cvarsoundlimit)) && (LastSound[client] < thetime)){
		SndCount[client] = (SndCount[client] + 1);
		LastSound[client] = thetime + waitTime;
		new playersconnected;
		playersconnected = GetMaxClients();
		for (new i = 1; i <= playersconnected; i++){
			if(IsClientInGame(i)){
				if(SndOn[i]){
					EmitSoundToClient(i, filelocation);
				}
			}
		}
	}

	if ((SndCount[client]) >= GetConVarInt(cvarsoundlimit)){
		PrintToChat(client,"[Funny Sounds] Sorry you have reached your sound quota!");
	}else if ((SndCount[client]) == GetConVarInt(cvarsoundwarn)){
		new numberleft;
		numberleft = (GetConVarInt(cvarsoundlimit) - GetConVarInt(cvarsoundwarn));
		PrintToChat(client,"[Funny Sounds] You only have %d sounds left!",numberleft);
	}
	return Plugin_Handled;
}

public Action:Command_Sound_Reset(client, args){
	if (args < 1)
	{
		ReplyToCommand(client, "[Funny Sounds] Usage: sm_sound_reset <user | all> : Resets sound quota for user, or everyone if all");
		return Plugin_Handled;	
	}
	new String:arg[32];
	GetCmdArg(1, arg, sizeof(arg));	
	
	if(strcmp(arg,"all",false) == 0 ){
		for (new i = 1; i <= MAX_PLAYERS; i++)
			SndCount[i] = 0;
		ReplyToCommand(client, "[Funny Sounds] Quota has been reset for all players");	
	}else{
		new user[2];
		new numplayer = SearchForClients(arg, user, 2);
		
		if (numplayer == 0){
			ReplyToCommand(client, "[Funny Sounds] No matching client");
			return Plugin_Handled;
		}else if (numplayer > 1){
			ReplyToCommand(client, "[Funny Sounds] More than one client matches");
			return Plugin_Handled;
		}else if ((client != 0) && (!CanUserTarget(client, user[0]))){
			ReplyToCommand(client, "[Funny Sounds] Unable to target");
			return Plugin_Handled;
		}else if (IsFakeClient(user[0])){
			ReplyToCommand(client, "[Funny Sounds] Cannot target a bot");
			return Plugin_Handled;
		}
			
		SndCount[user[0]] = 0;
		new String:clientname[64];
		GetClientName(user[0],clientname,MAX_PLAYERS);
		ReplyToCommand(client, "[Funny Sounds] Quota has been reset for %s", clientname);
	}
	return Plugin_Handled;
}

public Action:Command_Sound_Ban(client, args){
	if (args < 1)
	{
		ReplyToCommand(client, "[Funny Sounds] Usage: sm_sound_ban <user> : Bans a player from using sounds");
		return Plugin_Handled;	
	}
	new String:arg[32];
	GetCmdArg(1, arg, sizeof(arg));	
	
	new user[2];
	new numplayer = SearchForClients(arg, user, 2);
	
	if (numplayer == 0){
		ReplyToCommand(client, "[Funny Sounds] No matching client");
		return Plugin_Handled;
	}else if (numplayer > 1){
		ReplyToCommand(client, "[Funny Sounds] More than one client matches");
		return Plugin_Handled;
	}else if ((client != 0) && (!CanUserTarget(client, user[0]))){
		ReplyToCommand(client, "[Funny Sounds] Unable to target");
		return Plugin_Handled;
	}else if (IsFakeClient(user[0])){
		ReplyToCommand(client, "[Funny Sounds] Cannot target a bot");
		return Plugin_Handled;
	}
	
	new String:BanClient2[64];
	GetClientName(user[0],BanClient2,MAX_PLAYERS);
	
	if (restrict_playing_sounds[user[0]] == 1){
		ReplyToCommand(client, "[Funny Sounds] %s is already banned!", BanClient2);
	}else{
		restrict_playing_sounds[user[0]]=1;
		ReplyToCommand(client,"[Funny Sounds] %s has been banned!", BanClient2);
	}

	return Plugin_Handled;
}

public Action:Command_Sound_Unban(client, args){
	if (args < 1)
	{
		ReplyToCommand(client, "[Funny Sounds] Usage: sm_sound_unban <user> <1|0> : Unbans a player from using sounds");
		return Plugin_Handled;	
	}
	new String:arg[32];
	GetCmdArg(1, arg, sizeof(arg));	
	
	new user[2];
	new numplayer = SearchForClients(arg, user, 2);
	
	if (numplayer == 0){
		ReplyToCommand(client, "[Funny Sounds] No matching client");
		return Plugin_Handled;
	}else if (numplayer > 1){
		ReplyToCommand(client, "[Funny Sounds] More than one client matches");
		return Plugin_Handled;
	}else if ((client != 0) && (!CanUserTarget(client, user[0]))){
		ReplyToCommand(client, "[Funny Sounds] Unable to target");
		return Plugin_Handled;
	}else if (IsFakeClient(user[0])){
		ReplyToCommand(client, "[Funny Sounds] Cannot target a bot");
		return Plugin_Handled;
	}
	
	new String:BanClient2[64];
	GetClientName(user[0],BanClient2,MAX_PLAYERS);
	
	if (restrict_playing_sounds[user[0]] == 0){
		ReplyToCommand(client,"[Funny Sounds] %s is not banned!", BanClient2);
	}else{
		restrict_playing_sounds[user[0]]=0;
		ReplyToCommand(client,"[Funny Sounds] %s has been unbanned!", BanClient2);
	}
	return Plugin_Handled;
}


List_Sounds(client){
	KvRewind(listfile);
	KvJumpToKey(listfile, "ExitSound", false);
	KvGotoNextKey(listfile, true);
	decl String:buffer[255];
	do{
		KvGetSectionName(listfile, buffer, sizeof(buffer));
		PrintToConsole(client, buffer);
	} while (KvGotoNextKey(listfile));
	
}

public OnPluginEnd(){
  CloseHandle(listfile);
}